#define NOMINMAX

#include "CudaHelpers.h"

#include <algorithm>

extern "C" uint32_t device_warpsize (void)
{
    int devWarpSize;
    cudaDeviceGetAttribute(&devWarpSize, cudaDevAttrWarpSize, 0);
    return (uint32_t)devWarpSize;
}

// This method selects the block and thread counts, and the amount of data each ought to process.
// The algorithm here is considerably naive, and is likely only an optimal solution in select cases.
extern "C" void bufferDiffLaunchVariablesQWord (const uint32_t qwordsize, uint32_t *const qwordsdown, dim3 *const blockcnt, uint32_t *const threadcnt, dim3 *const blockqwordsize, uint32_t *const threadqwordsize)
{
    int maxBlocksPerDimX, maxBlocksPerDimY, maxThreadsPerBlock, devWarpSize;
    cudaDeviceGetAttribute(&maxBlocksPerDimX, cudaDevAttrMaxBlockDimX, 0);
    cudaDeviceGetAttribute(&maxBlocksPerDimY, cudaDevAttrMaxBlockDimY, 0);
    cudaDeviceGetAttribute(&maxThreadsPerBlock, cudaDevAttrMaxThreadsPerBlock, 0);
    cudaDeviceGetAttribute(&devWarpSize, cudaDevAttrWarpSize, 0);

    const uint32_t maxBlocks = maxBlocksPerDimX * maxBlocksPerDimY;

    if (!qwordsize)
    {
        // We can't handle data-blocks smaller than a QWORD.
        *blockcnt        = 0;
        *threadcnt       = 0;
        *blockqwordsize  = 0;
        *threadqwordsize = 0;
        *qwordsdown      = 0;
        return;
    }

    // First guess for thread and block size.
    uint32_t naiveThreadcnt = std::min<uint32_t>(maxThreadsPerBlock, qwordsize);
    while (0 != qwordsize % naiveThreadcnt && naiveThreadcnt > (uint32_t)devWarpSize)
        naiveThreadcnt /= 2;

    const uint32_t naiveBlockcnt = std::min<uint32_t>(maxBlocks, qwordsize / naiveThreadcnt);

    *threadcnt = naiveThreadcnt;

    *blockcnt = dim3(naiveBlockcnt);
    if (naiveBlockcnt > (uint32_t)maxBlocksPerDimX)
    {
        blockcnt->y = naiveBlockcnt / maxBlocksPerDimX;
        blockcnt->x = naiveBlockcnt / blockcnt->y;
    }

        // Determine the per-thread QWORD size (may lose precision; that's expected).
    *threadqwordsize    = qwordsize / naiveBlockcnt / naiveThreadcnt;
    
        // Determine how many bytes we're processing.
    *qwordsdown         = (*threadqwordsize) * naiveBlockcnt * naiveThreadcnt;
        // Determine the per-block QWORD size.
    blockqwordsize->y   = (*qwordsdown) / blockcnt->y;
    blockqwordsize->x   = blockqwordsize->y / blockcnt->x;
}
