#ifndef INCLUDED_QUADRANT_CHANNEL_GAIN_H
#define INCLUDED_QUADRANT_CHANNEL_GAIN_H

#include <cstdint>
// A pair of pixel gain numerators, where the denominator is 256u. These are therefore
// exponential gains (EG), with the linear gain (LG) being:
//      LG = log(1 + EG) / log(256) / 2
// and the EG being:
//      EG = 256 ^ (2 * LG) - 1
typedef std::uint16_t ChannelGain8;

struct PairGains8
{
    ChannelGain8 x[2];
};

struct QuadrantGains8
{
    PairGains8 y[2];
};

// Ensure that the given linear gain value is in its valid range.
extern "C" double       clampLinGain    (const double linGain);

// Ensure that the given exponential gain value is in its valid range.
extern "C" ChannelGain8 clampExpGain    (const double expGain);

// Convert from linear gain (range of [0., 1.]) to exponential gain (range of [0, 65536)).
extern "C" ChannelGain8 lin2expGain8    (const double linGain);

// Determine the linear gain (range of [0., 1.]) for the given exponential gain (range of [0, 65536)).
extern "C" double       exp2linGain8    (const ChannelGain8 expGain);

// Fill the QuadrantGains for the given linear gains.
extern "C" void         fillQuadGains8  (QuadrantGains8 *const qg, const double lgY0X0, const double lgY0X1, const double lgY1X0, const double lgY1X1);

// Per-element image processing kernel, multiplying each input pixel by it's
// matching quadrant gain, then dividing by 255. The image width and height
// must be multiples of 16 bytes.
extern "C" void quadrantChannelGain8 ( const std::uint8_t *const devImgIn
                                     , std::uint8_t *const devImgOut
                                     , const std::uint32_t width
                                     , const std::uint32_t height
                                     , const QuadrantGains8 quadrantGains );

#endif // INCLUDED_QUADRANT_CHANNEL_GAIN_H
