#ifndef INCLUDED__B_F__D_V_P__DEF__H
#define INCLUDED__B_F__D_V_P__DEF__H

/* FILE:        BFDVPDef.h
 * DATE:        4/29/2014
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2014, BitFlow, Inc.
 * DESCRIPTION: BitFlow definitions for an API to interface with an NVIDIA GPU
 *              supporting the GPUDirect for Video protocol.
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <BFType.h>

// CUDA is considered a prerequisite.
#include <cuda.h>

enum _BFDVPInitFlags
{
    BFDVPInit_Default = 0   // Initialize the calling thread with default configurations.
};
typedef BFU32                   BFDVPInitFlags;

// System buffer types.
typedef PBFU32                  BFDVPSysBuf;
typedef BFDVPSysBuf*            BFDVPSysBufArray;

// DVP API equivalents.
typedef BFU64                   BFDVPBufHandle;
typedef BFU64                   BFDVPSyncObjectHandle;

// System-device buffer pairing handle.
typedef struct _BFDVPSysDevPair
{
    BFDVPSysBuf                 sysBuffer;              // The system buffer pointer.
    CUdeviceptr                 devBuffer;              // The CUDA device buffer pointer.
 
    BFDVPBufHandle              sysDvpBuffer;           // DVP system buffer handle.
    PBFU8                       sysDvpSemaphoreBase;    // DVP system semaphore base allocation.
    volatile BFU32              *sysDvpSemaphore;       // DVP system semaphore.
    BFU32                       sysDvpAcquire;          // DVP system semaphore acquire handle.
    BFU32                       sysDvpRelease;          // DVP system semaphore release handle.
    BFDVPSyncObjectHandle       sysDvpSync;             // DVP system synchronization handle.
    
    BFDVPBufHandle              devDvpBuffer;           // DVP device buffer handle.
    PBFU8                       devDvpSemaphoreBase;    // DVP device semaphore base allocation.
    volatile BFU32              *devDvpSemaphore;       // DVP device semaphore.
    BFU32                       devDvpAcquire;          // DVP device semaphore acquire handle.
    BFU32                       devDvpRelease;          // DVP device semaphore release handle.
    BFDVPSyncObjectHandle       devDvpSync;             // DVP device synchronization handle.
    
    BFU32                       bufferSize;             // The buffer size in bytes.
    BFU32                       physicalSize;           // The physical buffer size in bytes.
} BFDVPSysDevPair, *PBFDVPSysDevPair;

// Allocation additional parameters.
typedef struct _BFDVPAllocParams
{
    BFBOOL                      keepWorkingSetSize;     // If TRUE, the allocator will not attempt to expand the process working set size.
} BFDVPAllocParams, *PBFDVPAllocParams;

// DVP DMA transfer additional parameters.
typedef struct _BFDVPTransferParams
{
    BFU64 ns_timeout;

    BFU32 sysBufOffset;
    BFU32 devBufOffset;
    BFU32 bufSize;
} BFDVPTransferParams, *PBFDVPTransferParams;

// BFDVP Integer Constants.
enum
{
    BFDVPTimeoutInfinite = ~BFU64(0),
    BFDVPBufSizeAll = ~BFU32(0)
};

#ifdef __cplusplus
}
#endif

#endif // INCLUDED__B_F__D_V_P__DEF__H
