#ifndef INCLUDED_B_F__D_V_P__ERROR__H
#define INCLUDED_B_F__D_V_P__ERROR__H

/* FILE:        BFDVPError.h
 * DATE:        4/29/2014
 * AUTHOR:      Jeremy Greene
 * COMPANY:     BitFlow, Inc.
 * COPYRIGHT:   Copyright (C) 2014, BitFlow, Inc.
 * DESCRIPTION: Error codes returned by the BFDVP API.
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <BFTabError.h>

typedef enum _BFDVPRC_Enum
{
    BFDVP_OK = BF_OK,

    BFDVP_ERROR,                // Generic or internal error.

    BFDVP_INVALID_HANDLE,       // A handle argument is null-pointer or otherwise invalid
    BFDVP_INVALID_BUFFER,       // A buffer argument is null-pointer or otherwise invalid
    BFDVP_INVALID_PARAMETER,    // A parameters argument is an invalid value.

    BFDVP_OUT_OF_MEMORY,        // A memory allocation failed.
    BFDVP_SYSTEM_ERROR,         // A system error occured during execution of the method. See GetLastError.
    BFDVP_DVP_ERROR,            // A DVP error occured during execution of the method. See debug messages.
    BFDVP_CUDA_ERROR,           // A CUDA error occured during execution of the method. See cudaGetLastError.
    BFDVP_UNKNOWN_ERROR,        // Any kind of unknown error.

    BFDVP_TIMEOUT_ERROR,        // A timeout occured during while performing the operation.

    BFDVP_ALREADY_INIT,         // BFDVPInit was called previously without a subsequent call to BFDVPDeinit.
    BFDVP_NOT_INIT              // BFDVPInit has not been called.
} BFDVPRC_Enum;

#ifdef __cplusplus
}
#endif

#endif // INCLUDED_B_F__D_V_P__ERROR__H
